#' Find Local Peaks
#'
#' Find local maxima in a neighbourhood. To find local minima, just provide
#' \code{-x} instead of \code{x}.
#'
#' @param x numeric vector.
#' @param m size of the neighbourhood.
#'
#' @return A vector of indices.
#'
#' @references Based on \url{https://github.com/stas-g/findPeaks}.
#'
#' @export
find_peaks <- function(x, m = 3) {
  shape <- diff(sign(diff(x, na.pad = FALSE)))
  unlist(sapply(which(shape < 0), function(i) {
    z <- i - m + 1
    z[z <= 0] <- 1
    w <- i + m + 1
    w[w >= length(x)] <- length(x)
    if(all(x[c(z : i, (i + 2) : w)] <= x[i + 1]))
      return(i + 1)
    return(numeric(0))
  }))
}

# Find Tutti
#
# Find tutti passages (loud passages where many instruments are playing at the
# same time) in an audio file based on a rolling average.
#
# @param file file path to an MP3 or WAV audio file.
# @param window window size, in seconds, to compute the rolling average.
# @param min.dur minimum tutti duration, in seconds.
# @param plot whether to produce a diagnostic plot.
# @param print print to standard output instead of returning a data frame.
#
# @return A data frame with start time and duration in seconds.
#
# @export
find_tutti <- function(file, window=5, min.dur=2*window, plot=FALSE, print=FALSE) {
  pcm <- switch(
    tolower(tools::file_ext(file)),
    "wav" = tuneR::readWave(file),
    "mp3" = tuneR::readMP3(file),
    stop("File format not supported"))

  pcm <- tuneR::mono(pcm, which="both")
  pcm <- tuneR::normalize(pcm)
  pcm.original <- pcm
  pcm <- pcm^2
  pcm@left <- zoo::rollmeanr(pcm@left, window*pcm@samp.rate)
  pcm <- tuneR::normalize(pcm, center=FALSE)

  threshold <- mean(pcm@left)

  ind <- which(pcm@left > threshold)
  dif <- c(ind[1], diff(ind))
  up <- ind[which(dif > 1)]

  ind <- which(pcm@left < threshold)
  dif <- c(ind[1], diff(ind))
  down <- ind[which(dif > 1)]

  if (length(up) > length(down))
    down <- c(down, length(pcm@left))
  if (length(down) > length(up))
    down <- utils::tail(down, -1)

  df <- data.frame(start=up, end=down)
  df$duration <- df$end - df$start
  df$int <- apply(df, 1, function(i) sum(pcm@left[i["start"]:i["end"]]))
  df$int <- df$int / df$duration
  df[, 1:3] <- df[, 1:3] / pcm@samp.rate
  df <- df[df$duration > min.dur, ]

  if (plot) {
    plot(pcm.original[1:length(pcm)], col="gray")
    for (i in seq_len(nrow(df))) {
      xlim <- c(df$start[i], df$end[i])
      ylim <- c(0, df$int[i])
      graphics::polygon(c(xlim, rev(xlim)), rep(ylim, each=2), col="blue", border=NA)
    }
    graphics::par(new=TRUE)
    plot(pcm, col="red", axes=FALSE, xlab="")
  }

  if (print) for (i in seq_len(nrow(df)))
    cat(df$start[i], df$duration[i], "\n")
  else df[, c("start", "duration")]
}

# check parity
is.odd <- function(x) x %% 2 != 0

# double factorial
factorial2 <- function(x) {
  odd <- is.odd(x)
  x[odd] <- (x[odd] + 1) / 2
  x[odd] <- gamma(0.5 + x[odd]) * 2^x[odd] / sqrt(pi)
  x[!odd] <- x[!odd] / 2
  x[!odd] <- 2^x[!odd] * factorial(x[!odd])
  x
}

# https://stackoverflow.com/a/4752580/6788081
# Determine if range of vector is FP 0.
zero_range <- function(x, tol = .Machine$double.eps ^ 0.5) {
  if (length(x) == 1) return(TRUE)
  x <- range(x) / mean(x)
  isTRUE(all.equal(x[1], x[2], tolerance = tol))
}
