## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(fig.align="center")

fig1 <- "**Mechanical metronomes.** From left to right, a contemporary Neewer© NW-707, TB 06, TB 07 (both from Tony Bingham's _Metronomes and Musical Time_), Maelzel's patent, and a diagram depicting the main parameters of the model."

## -----------------------------------------------------------------------------
library(bmetr)
library(errors)
options(errors.warn.coercion=FALSE)
library(dplyr, warn.conflicts=FALSE)
library(ggplot2)
theme_set(ggthemes::theme_tufte(base_family="sans"))

## ---- echo=FALSE, fig.cap=fig1------------------------------------------------
knitr::include_graphics(system.file("extdata/metronomes.jpg", package="bmetr"))

## -----------------------------------------------------------------------------
f.raw <- tuneR::readWave(system.file("extdata/100.wav", package="bmetr"))
f.raw

## -----------------------------------------------------------------------------
f <- tuneR::normalize(f.raw)^2
f@left[f@left < 0.5] <- 0
peaks <- find_peaks(f@left, f@samp.rate * 0.05)

## -----------------------------------------------------------------------------
tuneR::plot(f.raw)
points(peaks/f@samp.rate, abs(f.raw@left[peaks]), col="red", pch=19)

## -----------------------------------------------------------------------------
period <- diff(peaks) / f@samp.rate
set_errors(60) / set_errors(mean(period), sd(period)/sqrt(length(period)))

## ---- eval=FALSE--------------------------------------------------------------
#  wav <- Sys.glob(paste0(PATH_WAVS, "/*"))
#  mark <- as.numeric(sub(".wav", "", basename(wav)))
#  
#  bpm <- do.call(c, lapply(wav, function(x) {
#    f <- tuneR::normalize(tuneR::readWave(x))^2
#    f@left[f@left < 0.5] <- 0
#    peaks <- find_peaks(f@left, f@samp.rate * 0.05)
#    period <- diff(peaks) / f@samp.rate
#    set_errors(60) / set_errors(mean(period), sd(period)/sqrt(length(period)))
#  }))

## -----------------------------------------------------------------------------
head(metr.neewer)

## ---- message=FALSE, warning=FALSE--------------------------------------------
# measurements for the masses
# M <- set_errors(30.8, 0.01)
# mu <- set_errors(3.8, 0.01)

# correction for L = R, as for the rest of the metronomes
metr.params[1,]$L <- metr.params[1,]$R
M <- set_errors(31.01, 0.01)
m <- set_errors(7.1, 0.01)
mu <- set_errors(3.59, 0.01)
M. <- M / m
mu. <- mu / m

# gravitational acceleration
g <- set_errors(9.807, 0.04)

# attach rcm, l, R, L, A
metr.params %>%
  filter(model == "Neewer") %>%
  unite_errors() %>%
  attach()

neewer <- metr.marks %>%
  filter(model == "Neewer") %>%
  left_join(metr.neewer, by="mark") %>%
  unite_errors()

comparative <- neewer %>%
  mutate(Model  = metr_model(r+rcm, R, M., l, L, mu., g, A)) %>%
  mutate(Forsen = metr_model(r+rcm, R, M., l, L, 0,   g, 0)) %>%
  gather_errors("series", "value", "Model", "Forsen", "bpm") %>%
  mutate(series = sub("Forsen", "Forsén et al. (2013)", series)) %>%
  mutate(series = sub("bpm", "Experimental", series)) %>%
  mutate(series = reorder(factor(series), -drop_errors(value)))

ggplot(comparative) +
  aes(mark, drop_errors(value), color=series) +
  ggthemes::geom_rangeframe(aes(y=drop_errors(bpm)), data=neewer, color="black") +
  geom_abline(slope=1, color="gray") + geom_point(size=.7) +
  geom_errorbar(aes(ymin=errors_min(value), ymax=errors_max(value)), size=.3) +
  geom_smooth(method="gam", size=.3) +
  labs(x="Metronome mark [bpm]", y="Oscillation frequency [bpm]", color=NULL) +
  theme(legend.position=c(0, 1), legend.justification=c(0, 1),
        axis.title.y=element_text(hjust=0.2), axis.title.x=element_text(hjust=0.5))

## ---- message=FALSE, warning=FALSE--------------------------------------------
corrections <- neewer %>%
  mutate(base     = metr_model(r+rcm, R, M., l, L, 0,   g, 0, 0)) %>%
  mutate(Rod      = metr_model(r+rcm, R, M., l, L, mu., g, 0, 0)) %>%
  mutate(Angle    = metr_model(r+rcm, R, M., l, L, 0,   g, A, 0)) %>%
  mutate(Friction = metr_model(r+rcm, R, M., l, L, 0,   g, 0, 0.5)) %>%
  mutate(Rod      = (Rod      - base) / base * 100) %>%
  mutate(Angle    = (Angle    - base) / base * 100) %>%
  mutate(Friction = (Friction - base) / base * 100) %>%
  gather_errors("series", "value", "Rod", "Angle", "Friction") %>%
  mutate(series = factor(series, levels=c("Friction", "Angle", "Rod")))

ggplot(corrections) +
  aes(mark, drop_errors(value), color=series) +
  ggthemes::geom_rangeframe(color="black") +
  geom_point(size=.7) +
  geom_errorbar(aes(ymin=errors_min(value), ymax=errors_max(value)), size=.3) +
  geom_smooth(method="gam", size=.3) +
  labs(x="Metronome mark [bpm]", y="Correction [%]", color="") +
  theme(legend.position=c(0.98, 0.58), legend.justification=c(1, 0))

## -----------------------------------------------------------------------------
metr <- metr.marks
metr[grep("Neewer", metr$model),]$mark <- rep(metr.neewer$bpm, 2)
metr <- metr %>%
  group_by(model) %>%
  slice(-n())

fit <- metr_fit(metr, metr.params) %>%
  unite_errors() %>%
  mutate(model = factor(sub("Neewer", "Control", model)))

pred <- metr_predict(fit) %>%
  cbind(metr[-1]) %>%
  unite_errors() %>%
  mutate(model = factor(sub("Neewer", "Control", model)))

r.squared <- sapply(fit$fit, function(x) summary(x)$adj.r.squared)
labels <- mapply(function(x, y) {
  bquote(.(x)*"," ~ R^2==.(y))
}, levels(pred$model), round(r.squared, 4))

ggplot(pred) +
  aes(drop_errors(r), y, color=reorder(model, drop_errors(r), min)) +
  ggthemes::geom_rangeframe(color="black") +
  geom_ribbon(aes(ymin=lwr, ymax=upr), alpha=.2, size=0) +
  geom_point(size=.7) +
  geom_errorbarh(aes(xmin=errors_min(r), xmax=errors_max(r)), height=2, size=.3) +
  geom_line(aes(y=fit), size=.3) +
  scale_color_discrete(labels=labels) +
  labs(x="r [mm]", y=expression(Omega^2), color=NULL) +
  theme(legend.position=c(1, 1), legend.justification=c(1, 1))

## -----------------------------------------------------------------------------
note <- data.frame(model="Control (true)", M.=M., mu.=mu.)

ggplot(fit) +
  aes(drop_errors(M.), drop_errors(mu.), color=reorder(model, -drop_errors(mu.))) +
  ggthemes::geom_rangeframe(color="black") +
  geom_point(size=.7) +
  geom_point(aes(shape=model), data=note, color="black", size=2) +
  geom_errorbarh(aes(xmin=errors_min(M.), xmax=errors_max(M.)), height=.005, size=.3) +
  geom_errorbar(aes(ymin=errors_min(mu.), ymax=errors_max(mu.)), width=.012, size=.3) +
  scale_shape_manual(name=NULL, values=8) +
  labs(x="Nondimensionalized lower mass", y="Nondimensionalized rod mass", color=NULL) +
  guides(color=guide_legend(order=1), shape=guide_legend(order=2)) +
  theme(legend.position=c(.95, .95), legend.justification=c(1, 1), legend.spacing.y=unit(-2, "mm"))

## -----------------------------------------------------------------------------
res <- as.data.frame(fit[3:5, 1:3])
res$model <- as.character(res$model)
res[4,] <- list("Beethoven's metronome (average)", mean(res$M.), mean(res$mu.))
res

